/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2016  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.36 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
Licensing information

Licensor:                 SEGGER Software GmbH
Licensed to:              ARM Ltd, 110 Fulbourn Road, CB1 9NJ Cambridge, UK
Licensed SEGGER software: emWin
License number:           GUI-00181
License model:            LES-SLA-20007, Agreement, effective since October 1st 2011 
Licensed product:         MDK-ARM Professional
Licensed platform:        ARM7/9, Cortex-M/R4
Licensed number of seats: -
----------------------------------------------------------------------
File        : LCD_X.c
Purpose     : Port routines for SAM4S 8-bit Interface
----------------------------------------------------------------------
*/

#include "GUI.h"
#include "LCD_X.h"

/*********************************************************************
*
*       Hardware configuration
*
**********************************************************************
*/

#include "sam4s.h"                      // Device header

/*--------------- Graphic LCD interface hardware definitions -----------------*/

/* Note: LCD /CS is CCS1                                                      */
#define LCD_BASE   (0x61000000UL)                              // LCD base address

#define LCD_REG8   (*((volatile unsigned char *)(LCD_BASE  ))) // LCD register address
#define LCD_DAT8   (*((volatile unsigned char *)(LCD_BASE+2))) // LCD data address

#define LCD_D0_D7       (0xFF)
#define LCD_RS          (PIO_PC19)
#define LCD_RD          (PIO_PC11)
#define LCD_WR          (PIO_PC8)
#define LCD_NCS1        (PIO_PC15)
#define LCD_BACKLIGHT   (PIO_PC13)


/*********************************************************************
*
*       Exported code
*
*********************************************************************
*/
/*********************************************************************
*
*       LCD_X_Init
*
* Purpose:
*   This routine should be called from your application program
*   to set port pins to their initial values
*/
void LCD_X_Init(void) {
  uint32_t i;

/* Configure the LCD Control pins --------------------------------------------*/
  PMC->PMC_WPMR  = 0x504D4300;                  /* Disable write protect      */
  PMC->PMC_PCER1 = (1 << ID_PIOC);              /* Enable clock for PIOC      */

/* Configure the Data Bus pins -----------------------------------------------*/
  PIOC->PIO_PDR    =  (LCD_D0_D7);              /* Enable Peripheral function */
  PIOC->PIO_IDR    =  (LCD_D0_D7);              /* Disable interrupt          */
  PIOC->PIO_ABCDSR[0]  &= ~(LCD_D0_D7);         /* Select Peripheral A        */
  PIOC->PIO_ABCDSR[1]  &= ~(LCD_D0_D7);         /* Select Peripheral A        */

/* Configure the Control pins ------------------------------------------------*/
  PIOC->PIO_PDR    =  (LCD_RS);                 /* Enable Peripheral function */
  PIOC->PIO_IDR    =  (LCD_RS);                 /* Disable interrupt          */
  PIOC->PIO_ABCDSR[0] &= ~(LCD_RS);             /* Select Peripheral A        */
  PIOC->PIO_ABCDSR[1] &= ~(LCD_RS);             /* Select Peripheral A        */

  PIOC->PIO_PDR    =  (LCD_RD | LCD_WR);        /* Enable Peripheral function */
  PIOC->PIO_IDR    =  (LCD_RD | LCD_WR);        /* Disable interrupt          */
  PIOC->PIO_ABCDSR[0] &= ~(LCD_RD | LCD_WR);    /* Select Peripheral A        */
  PIOC->PIO_ABCDSR[1] &= ~(LCD_RD | LCD_WR);    /* Select Peripheral A        */

  PIOC->PIO_PDR    =  (LCD_NCS1);               /* Enable Peripheral function */
  PIOC->PIO_IDR    =  (LCD_NCS1);               /* Disable interrupt          */
  PIOC->PIO_ABCDSR[0] &= ~(LCD_NCS1);           /* Select Peripheral A        */
  PIOC->PIO_ABCDSR[1] &= ~(LCD_NCS1);           /* Select Peripheral A        */

/* Configure the Backlight pins ----------------------------------------------*/
  PIOC->PIO_IDR  =                              /* Disable interrupt          */
  PIOC->PIO_OER  =                              /* Enable output              */
  PIOC->PIO_PER  = (LCD_BACKLIGHT);             /* Disable Peripheral function*/
  PIOC->PIO_CODR = (LCD_BACKLIGHT);             /* Backlight off              */

/*-- SMC Configuration -------------------------------------------------------*/
/*--------------------------- CS2 --------------------------------------------*/
  PMC->PMC_PCER0 = (1 << ID_SMC);               /* Enable clock for SMC       */

  SMC->SMC_CS_NUMBER[1].SMC_SETUP = (SMC_SETUP_NWE_SETUP(2)    |
                                     SMC_SETUP_NCS_WR_SETUP(2) |
                                     SMC_SETUP_NRD_SETUP(2)    |
                                     SMC_SETUP_NCS_RD_SETUP(2)  );

  SMC->SMC_CS_NUMBER[1].SMC_PULSE = (SMC_PULSE_NWE_PULSE(4)    |
                                     SMC_PULSE_NCS_WR_PULSE(4) |
                                     SMC_PULSE_NRD_PULSE(10)   |
                                     SMC_PULSE_NCS_RD_PULSE(10) );

  SMC->SMC_CS_NUMBER[1].SMC_CYCLE = (SMC_CYCLE_NWE_CYCLE(10) |
                                     SMC_CYCLE_NRD_CYCLE(22)  );

  SMC->SMC_CS_NUMBER[1].SMC_MODE = ((SMC_MODE_READ_MODE) |
                                    (SMC_MODE_WRITE_MODE) );


  /* Turn backlight on with a certain backlight level                         */
  for (i = 0; i < 25 /* level */; i++) {
    PIOC->PIO_SODR = (LCD_BACKLIGHT);
    PIOC->PIO_SODR = (LCD_BACKLIGHT);
    PIOC->PIO_SODR = (LCD_BACKLIGHT);

    PIOC->PIO_CODR = (LCD_BACKLIGHT);
    PIOC->PIO_CODR = (LCD_BACKLIGHT);
    PIOC->PIO_CODR = (LCD_BACKLIGHT);
  }
  PIOC->PIO_SODR = (LCD_BACKLIGHT);
}

/*********************************************************************
*
*       LCD_X_Write0_8
*
* Purpose:
*   Write to controller, with A0 = 0
*/
void LCD_X_Write0_8(U8 c) {
  LCD_REG8 = c;
}

/*********************************************************************
*
*       LCD_X_Read0_8
*
* Purpose:
*   Read from controller, with A0 = 0
*/
U8 LCD_X_Read0_8(void) {
  return (LCD_REG8);
}

/*********************************************************************
*
*       LCD_X_Write1_8
*
* Purpose:
*   Write to controller, with A0 = 1
*/
void LCD_X_Write1_8(U8 c) {
  LCD_DAT8 = c;
}

/*********************************************************************
*
*       LCD_X_Read1_8
*
* Purpose:
*   Read from controller, with A0 = 1
*/
U8 LCD_X_Read1_8(void) {
  return (LCD_DAT8);
}

/*********************************************************************
*
*       LCD_X_WriteM1_8
*
* Purpose:
*   Write multiple bytes to controller, with A0 = 1
*/
void LCD_X_WriteM1_8(U8 * pData, int NumItems) {
  for (; NumItems; NumItems--) {
    LCD_DAT8 = *pData++;
  }
}

/*********************************************************************
*
*       LCD_X_ReadM1_8
*
* Purpose:
*   Read multiple bytes from controller, with A0 = 1
*/
void LCD_X_ReadM1_8(U8 * pData, int NumItems) {
  volatile U8 dummy;

  if (NumItems & 1) {
    dummy = LCD_DAT8;            // work around for emWin bug
  }
  for (; NumItems; NumItems--) {
    *pData++ = LCD_DAT8;
  }
}

/*************************** End of file ****************************/
